@@ -34,20 +34,25 @@ impl PgGuardRewriter {
     pub fn item_fn(
         &self,
         func: ItemFn,
+        entity_submission: Option<&pgx_utils::sql_entity_graph::PgExtern>,
         rewrite_args: bool,
         is_raw: bool,
         no_guard: bool,
     ) -> (proc_macro2::TokenStream, bool) {
         if rewrite_args {
-            self.item_fn_with_rewrite(func, is_raw, no_guard)
+            self.item_fn_with_rewrite(func, entity_submission, is_raw, no_guard)
         } else {
-            (self.item_fn_without_rewrite(func, no_guard), true)
+            (
+                self.item_fn_without_rewrite(func, entity_submission, no_guard),
+                true,
+            )
         }
     }
 
     fn item_fn_with_rewrite(
         &self,
         mut func: ItemFn,
+        entity_submission: Option<&pgx_utils::sql_entity_graph::PgExtern>,
         is_raw: bool,
         no_guard: bool,
     ) -> (proc_macro2::TokenStream, bool) {
@@ -102,6 +107,7 @@ impl PgGuardRewriter {
                     generics,
                     func_call,
                     rewritten_return_type,
+                    entity_submission,
                     no_guard,
                 ),
                 true,
@@ -118,6 +124,7 @@ impl PgGuardRewriter {
                     func_name_wrapper,
                     generics,
                     func_call,
+                    entity_submission,
                     false,
                 ),
                 true,
@@ -132,6 +139,7 @@ impl PgGuardRewriter {
                     func_name_wrapper,
                     generics,
                     func_call,
+                    entity_submission,
                     true,
                 ),
                 true,
@@ -146,6 +154,7 @@ impl PgGuardRewriter {
                     func_name_wrapper,
                     generics,
                     func_call,
+                    entity_submission,
                     false,
                 ),
                 true,
@@ -160,6 +169,7 @@ impl PgGuardRewriter {
                     func_name_wrapper,
                     generics,
                     func_call,
+                    entity_submission,
                     true,
                 ),
                 true,
@@ -175,6 +185,7 @@ impl PgGuardRewriter {
         generics: &Generics,
         func_call: proc_macro2::TokenStream,
         rewritten_return_type: proc_macro2::TokenStream,
+        sql_graph_entity_submission: Option<&pgx_utils::sql_entity_graph::PgExtern>,
         no_guard: bool,
     ) -> proc_macro2::TokenStream {
         let guard = if no_guard {
@@ -182,6 +193,7 @@ impl PgGuardRewriter {
         } else {
             quote! {#[pg_guard]}
         };
+        let sql_graph_entity_submission = sql_graph_entity_submission.cloned().into_iter();
         quote_spanned! {func_span=>
             #prolog
 
@@ -194,6 +206,8 @@ impl PgGuardRewriter {
 
                 #rewritten_return_type
             }
+
+            #(#sql_graph_entity_submission)*
         }
     }
 
@@ -208,6 +222,7 @@ impl PgGuardRewriter {
         func.sig.output = ReturnType::Default;
         let sig = func.sig;
         let body = func.block;
+        // We do **not** put an entity submission here as there still exists a `pg_extern` attribute.
         quote_spanned! {func_span=>
             #[pg_extern]
             #sig -> #return_type {
@@ -224,9 +239,12 @@ impl PgGuardRewriter {
         func_name_wrapper: Ident,
         generics: &Generics,
         func_call: proc_macro2::TokenStream,
+        sql_graph_entity_submission: Option<&pgx_utils::sql_entity_graph::PgExtern>,
         optional: bool,
     ) -> proc_macro2::TokenStream {
         let generic_type = proc_macro2::TokenStream::from_str(types.first().unwrap()).unwrap();
+        let mut generic_type = syn::parse2::<syn::Type>(generic_type).unwrap();
+        pgx_utils::anonymonize_lifetimes(&mut generic_type);
 
         let result_handler = if optional {
             quote! {
@@ -244,6 +262,8 @@ impl PgGuardRewriter {
             }
         };
 
+        let sql_graph_entity_submission = sql_graph_entity_submission.cloned().into_iter();
+
         quote_spanned! {func_span=>
             #prolog
             #[pg_guard]
@@ -293,6 +313,8 @@ impl PgGuardRewriter {
                     },
                 }
             }
+
+            #(#sql_graph_entity_submission)*
         }
     }
 
@@ -304,6 +326,7 @@ impl PgGuardRewriter {
         func_name_wrapper: Ident,
         generics: &Generics,
         func_call: proc_macro2::TokenStream,
+        entity_submission: Option<&pgx_utils::sql_entity_graph::PgExtern>,
         optional: bool,
     ) -> proc_macro2::TokenStream {
         let numtypes = types.len();
@@ -327,6 +350,8 @@ impl PgGuardRewriter {
 
         let composite_type = format!("({})", types.join(","));
         let generic_type = proc_macro2::TokenStream::from_str(&composite_type).unwrap();
+        let mut generic_type = syn::parse2::<syn::Type>(generic_type).unwrap();
+        pgx_utils::anonymonize_lifetimes(&mut generic_type);
 
         let result_handler = if optional {
             quote! {
@@ -343,6 +368,7 @@ impl PgGuardRewriter {
                 let result = pgx::PgMemoryContexts::For(funcctx.multi_call_memory_ctx).switch_to(|_| { #func_call result });
             }
         };
+        let sql_graph_entity_submission = entity_submission.cloned().into_iter();
 
         quote_spanned! {func_span=>
             #prolog
@@ -402,12 +428,15 @@ impl PgGuardRewriter {
                     },
                 }
             }
+
+            #(#sql_graph_entity_submission)*
         }
     }
 
     fn item_fn_without_rewrite(
         &self,
         mut func: ItemFn,
+        entity_submission: Option<&pgx_utils::sql_entity_graph::PgExtern>,
         no_guard: bool,
     ) -> proc_macro2::TokenStream {
         // remember the original visibility and signature classifications as we want
@@ -451,6 +480,8 @@ impl PgGuardRewriter {
             }
         };
 
+        let sql_graph_entity_submission = entity_submission.cloned().into_iter();
+
         quote_spanned! {func.span()=>
             #prolog
             #vis #sig {
@@ -458,6 +489,8 @@ impl PgGuardRewriter {
                 #func
                 #func_call
             }
+
+            #(#sql_graph_entity_submission)*
         }
     }
 
@@ -678,19 +711,22 @@ impl FunctionSignatureRewriter {
                 FnArg::Typed(ty) => match ty.pat.deref() {
                     Pat::Ident(ident) => {
                         let name = Ident::new(&format!("{}_", ident.ident), ident.span());
-                        let type_ = &ty.ty;
-                        let is_option = type_matches(type_, "Option");
+                        let mut type_ = ty.ty.clone();
+                        let is_option = type_matches(&type_, "Option");
 
                         if have_fcinfo {
                             panic!("When using `pg_sys::FunctionCallInfo` as an argument it must be the last argument")
                         }
 
                         let ts = if is_option {
-                            let option_type = extract_option_type(type_);
+                            let option_type = extract_option_type(&type_);
+                            let mut option_type = syn::parse2::<syn::Type>(option_type).unwrap();
+                            pgx_utils::anonymonize_lifetimes(&mut option_type);
+
                             quote_spanned! {ident.span()=>
                                 let #name = pgx::pg_getarg::<#option_type>(fcinfo, #i);
                             }
-                        } else if type_matches(type_, "pg_sys :: FunctionCallInfo") {
+                        } else if type_matches(&type_, "pg_sys :: FunctionCallInfo") {
                             have_fcinfo = true;
                             quote_spanned! {ident.span()=>
                                 let #name = fcinfo;
@@ -700,6 +736,7 @@ impl FunctionSignatureRewriter {
                                 let #name = pgx::pg_getarg_datum_raw(fcinfo, #i) as #type_;
                             }
                         } else {
+                            pgx_utils::anonymonize_lifetimes(&mut type_);
                             quote_spanned! {ident.span()=>
                                 let #name = pgx::pg_getarg::<#type_>(fcinfo, #i).unwrap_or_else(|| panic!("{} is null", stringify!{#ident}));
                             }
